#pragma once

#include "debug.hpp"

#include <stdint.h>
#include <sstream>
#include <boost/date_time.hpp>
#include <boost/mpl/vector.hpp>

#pragma pack(1)

namespace knx
{

enum dpt_index
{
    DPT_1_001,
    DPT_2_001,
    DPT_3_007,
    DPT_4_001,
    DPT_5_001,
    DPT_6_001,
    DPT_9_001,
    DPT_10_001,
};

class dpt
{
public:
    template<typename data_type>
    static std::string describe ( const typename data_type::major_type::set_type & data );
};

/**
 * @brief major class for all 1.XXX data point types
 */
class KNX_IMPORT_EXPORT dpt_1
{
public:
    typedef uint8_t data_type;
    typedef bool set_type;
    static void set ( set_type value, data_type & data );
    static set_type get ( const data_type & data );
};

/**
 * \brief data point type 1.001 simple boolean value
 */
class KNX_IMPORT_EXPORT dpt_1_001
{
public:
    static const int INDEX = DPT_1_001;
    typedef dpt_1 major_type;
    typedef TYPENAME major_type::set_type set_type;
    static const major_type::set_type ON;
    static const major_type::set_type OFF;
    static std::string describe ( const major_type::set_type & data );
};

//! data point type 1.001 is better known as switch
typedef dpt_1_001 dpt_switch;

/**
 * @brief major class for all 2.XXX data point types
 */
class KNX_IMPORT_EXPORT dpt_2
{
public:
    typedef uint8_t data_type;
    typedef std::pair<bool,bool> set_type;
    static void set ( set_type value, data_type & data );
    static set_type get ( const data_type & data );
};

/**
 * \brief data point type 2.001 control value
 */
class KNX_IMPORT_EXPORT dpt_2_001
{
public:
    static const int INDEX = DPT_2_001;
    typedef dpt_2 major_type;
    typedef TYPENAME major_type::set_type set_type;
    static const major_type::set_type NO_CONTROL;
    static const major_type::set_type CONTROL_VALUE_ZERO;
    static const major_type::set_type CONTROL_VALUE_ONE;
    static std::string describe ( const major_type::set_type & data );
};

/**
 * @brief major class for all 3.XXX data point types
 */
class KNX_IMPORT_EXPORT dpt_3
{
public:
    typedef uint8_t data_type;
    typedef std::pair<bool,uint8_t> set_type;
    static void set ( set_type value, data_type & data );
    static set_type get ( const data_type & data );
};

/**
 * \brief data point type 3.007 dimming control
 */
class KNX_IMPORT_EXPORT dpt_3_007
{
public:
    static const int INDEX = DPT_3_007;
    typedef dpt_3 major_type;
    typedef TYPENAME major_type::set_type set_type;
    static std::string describe ( const major_type::set_type & data );
};

/**
 * @brief major class for all 4.XXX data point types
 */
class KNX_IMPORT_EXPORT dpt_4
{
public:
    class data
    {
    private:
        uint8_t static_part;
    public:
        data ( uint8_t character = 0 ) :static_part ( 0 ),character ( character ) {}
        uint8_t character;
    };
    typedef data data_type;
    typedef uint8_t set_type;
    static void set ( set_type value, data_type & data );
    static set_type get ( const data_type & data );
};

/**
 * \brief data point type 4.001 char ascii
 */
class KNX_IMPORT_EXPORT dpt_4_001
{
public:
    static const int INDEX = DPT_4_001;
    typedef dpt_4 major_type;
    typedef TYPENAME major_type::set_type set_type;
    static std::string describe ( const major_type::set_type & data );
};

/**
 * @brief major class for all 5.XXX data point types
 */
class dpt_5
{
public:
    class data
    {
    private:
        uint8_t static_part;
    public:
        data ( uint8_t unsigned_value = 0 );
        uint8_t unsigned_value;
    };
    typedef data data_type;
    typedef uint8_t set_type;
    static void set ( set_type value, data_type & data );
    static set_type get ( const data_type & data );
};

/**
 * \brief data point type 5.001 unsigned 8 bit
 */
class dpt_5_001
{
public:
    static const int INDEX = DPT_5_001;
    typedef dpt_5 major_type;
    typedef TYPENAME major_type::set_type set_type;
    static std::string describe ( const major_type::set_type & data );
};

//! 5.001 is better known as scaling value
typedef dpt_5_001 dpt_scaling;

/**
 * @brief major class for all 6.XXX signed 8 bit value
 */
class dpt_6
{
public:
    class data
    {
    private:
        uint8_t static_part;
    public:
        data ( int8_t real_signed_value = 0 ) :static_part ( 0 ),real_signed_value ( real_signed_value ) {}
        int8_t real_signed_value;
    };
    typedef data data_type;
    typedef int8_t set_type;
    static void set ( set_type value, data_type & data );
    static set_type get ( const data_type & data );
};

/**
 * \brief data point type 6.001 percent v8
 */
class dpt_6_001
{
public:
    static const int INDEX = DPT_6_001;
    typedef dpt_6 major_type;
    typedef TYPENAME major_type::set_type set_type;
    static std::string describe ( const major_type::set_type & data );
};

/**
 * @brief major class for all 9.XXX data point types
 */
class dpt_9
{
public:
	static const uint16_t INVALID_DATA = 0x7fff;
    class data
    {
    private:
        uint8_t static_part;
    public:
        data ( float float_value = 0.f );
        uint16_t value;
    };
    typedef data data_type;
    typedef float set_type;
    static void set ( set_type value, data_type & data );
    static set_type get ( const data_type & data );
};

/**
 * \brief data point type 9.001 value temp
 */
class dpt_9_001
{
public:
    static const int INDEX = DPT_9_001;
    typedef dpt_9 major_type;
    typedef TYPENAME major_type::set_type set_type;
    static std::string describe ( const major_type::set_type & data );
};

/**
 * @brief major class for all 10.XXX data point types
 */
class dpt_10
{
public:
    //! @brief internal class storing a time value
    class time
    {
    public:
        //! @brief creates a current time value
        time();
        uint8_t day;
        uint8_t hour;
        uint8_t minutes;
        uint8_t seconds;
        static const uint8_t NODAY = 0;
        static const uint8_t MONDAY = 1;
        static const uint8_t TUESDAY = 2;
        static const uint8_t WEDNESDAY = 3;
        static const uint8_t THURSDAY = 4;
        static const uint8_t FRIDAY = 5;
        static const uint8_t SATURDAY = 6;
        static const uint8_t SUNDAY = 7;
    };
    typedef uint32_t data_type;
    typedef time set_type;
    static void set ( set_type value, data_type & data );
    static set_type get ( const data_type & data );
};

/**
 * @brief class holding a time value
 */
class KNX_IMPORT_EXPORT dpt_10_001
{
public:
    static const int INDEX = DPT_10_001;
    typedef dpt_10 major_type;
    typedef TYPENAME major_type::set_type set_type;
    static std::string describe ( const major_type::set_type & data );
    static major_type::time now();
};

//! 10.001 is known as time of day value including hours. minutes, seconds and day of the week
typedef dpt_10_001 dpt_timeofday;

typedef boost::mpl::vector<
dpt_1_001,
dpt_2_001,
dpt_3_007,
dpt_4_001,
dpt_5_001,
dpt_6_001,
dpt_9_001,
dpt_10_001
> dpt_types;

}

#pragma pack()




